package cz.moneta.test.regression.backoffice.ufooperations.regressionsbbo;

import cz.moneta.test.dsl.Harness;
import cz.moneta.test.dsl.ufo.operations.search.SearchBy;
import cz.moneta.test.dsl.util.DateUtils;
import cz.moneta.test.dsl.util.credentials.Keys;
import cz.moneta.test.dsl.util.task.ufo.operations.OperationsTasks;
import cz.moneta.test.harness.annotations.JiraTestCase;
import cz.moneta.test.harness.annotations.ParameterizedTestCase;
import cz.moneta.test.harness.annotations.TestScenarioWithOrder;
import cz.moneta.test.harness.support.auth.Credentials;
import cz.moneta.test.harness.support.auth.Key;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.params.provider.ValueSource;

import static cz.moneta.test.dsl.util.task.ufo.cb.CommercialTasks.JIRA_COMMENT_STORE_KEY;
import static cz.moneta.test.harness.annotations.Environment.PPE;

@TestScenarioWithOrder(name = "Print Templates Uncategorized")
public class PrintTemplatesUncategorized {

    String department = "OPERATIONS_PAYMENTS_ZAKLAD";
    String category = "bez kategorie";
    String account;

    @Order(1)
    @ParameterizedTestCase(name = "Print template Uncategorized Another Address", environments = {PPE})
    @ValueSource(strings = {
            "Přijetí výpovědi",
            "Vyčíslení FLX",
            "Změna kontaktní adresy",
            "Změna trvalé adresy - odkaz na OM",
            "Změna kontaktní adresy - chybí podpis",
            "Změna příjmení",
            "Upozornění k Dětskému/Studentskému účtu"

    })
    @JiraTestCase(id = "BOC-T8677")
    public void printTemplatesUncategorizedAnotherAddress(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.account = "207293349";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        String printAlternateName = printName;
        if (printName.equals("Vyčíslení FLX")) {
            printAlternateName = "Vyčíslení Flexikreditu";
        }

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .fillSearchInput(account)
                .searchBy(SearchBy.ACCOUNT_NUMBER)
                .onClientDetailsPage()
                .clickPrintLettersWithAccount()
                .selectDepartment(department)
                .selectCategory(category)
                .selectTemplate(printName)
                .selectGenderIfVisible()
                .clickAnotherAddress()
                .fillNameInput("David Novák")
                .fillStreetInput("Dlouhá")
                .fillBuildingNumberInput("928")
                .fillStreetNumberInput("1")
                .fillCityInput("Praha")
                .fillPostalCodeInput("11000")
                .clickPrintWithAnotherAddress()
                .clickAddress()
                .clickPrint()
                .clickSaveToDmsIfNotChecked()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printAlternateName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(2)
    @ParameterizedTestCase(name = "Print template Uncategorized DMS", environments = {PPE})
    @ValueSource(strings = {
            "Přijetí výpovědi",
            "Vyčíslení FLX",
            "Změna kontaktní adresy",
            "Změna trvalé adresy - odkaz na OM",
            "Změna kontaktní adresy - chybí podpis",
            "Změna příjmení",
            "Upozornění k Dětskému/Studentskému účtu"

    })
    @JiraTestCase(id = "BOC-T8675")
    public void printTemplatesUncategorizedDms(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.account = "207118719";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        String printAlternateName = printName;
        if (printName.equals("Vyčíslení FLX")) {
            printAlternateName = "Vyčíslení Flexikreditu";
        }

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithAccount(account, department, category, printName))
                .clickSaveToDmsIfNotChecked()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printAlternateName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(3)
    @ParameterizedTestCase(name = "Print template Uncategorized Dopis", environments = {PPE})
    @ValueSource(strings = {
            "Přijetí výpovědi",
            "Vyčíslení FLX",
            "Změna kontaktní adresy",
            "Změna trvalé adresy - odkaz na OM",
            "Změna kontaktní adresy - chybí podpis",
            "Změna příjmení",
            "Upozornění k Dětskému/Studentskému účtu"

    })
    @JiraTestCase(id = "BOC-T8678")
    public void printTemplatesUncategorizedLetter(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.account = "162331005";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        String printAlternateName = printName;
        if (printName.equals("Vyčíslení FLX")) {
            printAlternateName = "Vyčíslení Flexikreditu";
        }

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithAccount(account, department, category, printName))
                .clickSaveToEmailIfChecked()
                .clickSaveToEts()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printAlternateName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(4)
    @ParameterizedTestCase(name = "Print template Uncategorized IB", environments = {PPE})
    @ValueSource(strings = {
            "Přijetí výpovědi",
            "Vyčíslení FLX",
            "Změna kontaktní adresy",
            "Změna trvalé adresy - odkaz na OM",
            "Změna příjmení",
            "Upozornění k Dětskému/Studentskému účtu"

    })
    @JiraTestCase(id = "BOC-T8681")
    public void printTemplatesUncategorizedIb(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.account = "235702963";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        String printAlternateName = printName;
        if (printName.equals("Vyčíslení FLX")) {
            printAlternateName = "Vyčíslení Flexikreditu";
        }

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithAccount(account, department, category, printName))
                .clickSaveToIbIfNotChecked()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printAlternateName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(5)
    @ParameterizedTestCase(name = "Print template Statements Interactive", environments = {PPE})
    @ValueSource(strings = {
            "Doplnění VS",
    })
    @JiraTestCase(id = "BOC-T8704")
    public void printTemplatesUncategorizedInteractive(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.account = "207019921";
        String title = "Test Klient";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .fillSearchInput(account)
                .searchBy(SearchBy.ACCOUNT_NUMBER)
                .onClientDetailsPage()
                .clickPrintLettersWithAccount()
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateInteractive(department, category, printName, title));
    }

    @Order(6)
    @ParameterizedTestCase(name = "Print template Uncategorized Local", environments = {PPE})
    @ValueSource(strings = {
            "Přijetí výpovědi",
            "Vyčíslení FLX",
            "Změna kontaktní adresy",
            "Změna trvalé adresy - odkaz na OM",
            "Změna kontaktní adresy - chybí podpis",
            "Změna příjmení",
            "Upozornění k Dětskému/Studentskému účtu"

    })
    @JiraTestCase(id = "BOC-T8680")
    public void printTemplatesUncategorizedLocal(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.account = "207246024";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithAccount(account, department, category, printName))
                .clickSaveToLocal()
                .clickSendAndClosePrintWindow();
    }

    @AfterEach
    protected void teardown(Harness harness) {
        harness.store(JIRA_COMMENT_STORE_KEY,
                "\nUFO Session ID: " + harness.get("reports.tmfj.execution.commentufosessionid") +
                        "\nNázev dopisu: " + harness.get("PRINT_NAME") +
                        "\nČíslo účtu: " + account);
    }
}